local ltn12 = require("ltn12")
local http = require("http")
function http.get(u)
   local t = {}
   local r, c, h = http.request{
      url = u,
      sink = ltn12.sink.table(t)}
   return r, c, h, table.concat(t)
end
function http.post(u, b)
    local t = {}
    local r, c, h = http.request{
        url = u,
        headers =
            {
                ["Content-Type"] = "application/x-www-form-urlencoded";
                ["Content-Length"] = #b;
            },
        method = "POST",
        source = ltn12.source.string(b),
        sink = ltn12.sink.table(t)}
    return r, c, h, table.concat(t)
end

function get_real_url(url, head, tail)
    res, code, headers, body = http.get(url)
    if code == 200 then
        local pos_h = string.find(body, head)
        local pos_b = string.len(head) + pos_h
        local pos_t = string.find(body, tail, pos_b)
        url = string.sub(body, pos_b, pos_t - 1)
        url_r, count = string.gsub(url, '\\', '')
        return url_r
    end

    return ""
end

function parse_res(res)
    local resObj = cjson.decode(res) 
    if resObj.type == "0" then
        return res
    else
        local head = resObj['start']
        local tail = resObj['end']
        local segs = resObj['seg']

        if segs ~= nil and head ~= nil or tail ~= nil then
            for format, format_seg in pairs(segs) do
                for k, v in pairs(format_seg) do
                    url = get_real_url(v['url'], head, tail)
                    if url ~= '' then
                        v['url'] = url
                    end
                end
            end
        end

        return cjson.encode(resObj)
    end
end


local ck_url = "http://suv.fun.tv/ck/v3"
--local ck_url = "http://111.161.35.199:7410/ck/v3"
function ck(para, uid)
    local res = is_pptv(para)
    if res ~= nil then
        return res
    else
        local u = string.format("%s?uid=%s", ck_url, uid)
        r, c, h, b = http.post(u, para)
        if c ~= 200 then
            return cjson.encode({a="0", b="0", c="0", d=cjson.encode({retcode=tostring(c), retmsg="crack failed"}), e="0"})
        end
        return b
    end
end

function kc(data)
    return parse_res(data)
end

function is_pptv(b)
    local taskObj = cjson.decode(b)
    if taskObj.site ~= "pptv" then
        return nil
    else
        errd= cjson.encode({retcode="404", retmsg="crack failed"})
        local res = cjson.encode({a="0", b="0", c="0", d=errd, e="0"})
        local site = taskObj["site"]
        local url = taskObj["url"]
        local vid = taskObj["vid"]
        local os = taskObj["os"]
        if url ~= nil then
            local pid = pptv_pid(url)
            if pid ~= nil then
                local info = pptv_info(pid)
                if info ~= nil then
                    res = pptv_res(info, vid, site, url, errd)
                end
            end
        end
        return res
    end 
end

function pptv_pid(u)
    local url = string.gsub(u, "v.pptv.com", "m.pptv.com")
    local res, code, headers, body = http.get(url)
    if code == 200 then
        local pid = string.match(body, "channel_id%p%p(%d+)")
        return pid
    else
        return nil
    end
end

function pptv_info(pid)
    local api_url = "http://web-play.pptv.com/webplay3-0-%s.xml?version=4&type=web.fpp&cb=getPlayEncode"
    local api = string.format(api_url, pid)
    local res, code, headers, body = http.get(api)
    if code == 200 then
        body = string.sub(body,15,-3)
        return body
    else
        return nil
    end 
end

function pptv_res(info, vid, site, o_url, errd)
    local switch = {
    F = {'fluent', 'high', 'super', 'original'}, -- del [4]= 'original_high'
    R = {}, -- rid
    H = {}, -- host
    K = {}, -- key
    S = {}, -- sgm
    err = {a="0", b="0", c="0", d=errd, e="0"},
    res = {['vid']=vid, ['site']=site, ['start']='', ['end']='', ['type']='0', ['url']=o_url, format={}, seg={}},
    entrance = function(self)
        local obj = cjson.decode(info)
        local ch = obj['childNodes']
        if ch ~= nil then
            for i, node in pairs(ch) do
                tag_name = node['tagName']
                if tag_name ~= nil then
                    func = self[tag_name]
                    if func ~= nil then
                        func(self, node)
                    end
                end
            end
            return self:exit()
        else
            return cjson.encode(self.err)
        end
    end,
    exit = function(self)
        for ft, r in pairs(self.R) do
            h = self.H[ft]
            k = self.K[ft]
            f = self.F[ft]
            if ft ~= nil and r ~= nil and h ~= nil and k ~= nil and f ~= nil then
                table.insert(self.res.format, f)
                seg_obj =  {}
                for i, d in pairs(self.S[ft]) do
                    rurl = string.format("http://%s/%s/%s?fpp.ver=1.3.0.19&k=%s&type=web.fpp", h, i-1, r, k)
                    table.insert(seg_obj, {['url']=rurl, ['duration']=d})
                end
                if next(seg_obj) then
                    self.res.seg[f] = seg_obj
                end
            end
        end
        if next(self.res.format) then
            self.err.d = cjson.encode(self.res)
        end
        return cjson.encode(self.err)
    end,
    channel = function(self, node)
        ch = node['childNodes']
        if ch ~= nil then
            for i, n in pairs(ch) do
                tag_name = n['tagName']
                if tag_name ~= nil then
                    func = self[tag_name]
                    if func ~= nil then
                        func(self, n)
                    end
                end
            end
        end
    end,
    dt = function(self, node)
        ch = node['childNodes']
        ft = node['ft']
        if ch ~= nil and ft ~= nil then
            ft = ft + 1
            for i, n in pairs(ch) do
                if n.tagName == 'sh' then
                    sh = n['childNodes'][1]
                    if sh ~= nil then
                        self.H[ft] = sh
                    end
                elseif n.tagName == 'key' then
                    k = n['childNodes'][1]
                    if k ~= nil then
                        self.K[ft] = k
                    end
                end
            end
        end
    end,
    dragdata = function(self, node)
        ch = node['childNodes']
        ft = node['ft']
        if ch ~= nil and ft ~= nil then
            ft = ft + 1
            self.S[ft] = {}
            for i, n in pairs(ch) do
                if n.tagName == 'sgm' then
                    dur = n['dur']
                    no = n['no']
                    if dur ~= nil and no ~= nil then
                        no = no + 1
                        self.S[ft][no] = tostring(math.ceil(dur) - 1)
                    end
                end
            end
        end
    end,
    file = function(self, node)
        ch = node['childNodes']
        if ch ~= nil then
            for i, n in pairs(ch) do
                ft = n['ft']
                rid = n['rid']
                if ft ~= nil and rid ~= nil then
                    ft = ft + 1
                    self.R[ft] = rid
                end
            end
        end
    end,
    }
    return switch:entrance()
end
